
// No AA
uniform sampler2D InputTexture;
uniform vec2 TargetSize;

layout(location = 0) out vec3 aa_out;

void main() {
	vec2 tex_coords = gl_FragCoord.xy / TargetSize;
	aa_out = texture(InputTexture, tex_coords).rgb;
}


/*

// Styves AA

uniform sampler2D InputTexture;
uniform vec2 TargetSize;

layout(location = 0) out vec3 aa_out;

float get_luminance(const in vec3 color) {
	return dot(color, vec3(0.2126, 0.7152, 0.0722));
}

const vec2 kernel[8] = vec2[8](
	vec2(-1.0, -1.0), // 0 - bl
	vec2(-1.0,  0.0), // 1 - l
	vec2(-1.0,  1.0), // 2 - tl
	vec2( 0.0, -1.0), // 3 - b
	vec2( 0.0,  1.0), // 4 - t
	vec2( 1.0, -1.0), // 5 - br
	vec2( 1.0,  0.0), // 6 - r
	vec2( 1.0,  1.0)  // 7 - tr
);

// distance between luminance samples
const float sample_scale = 2.0;

// distance between result blend samples
const float normal_scale = 1.0;

void main() {
	vec2 tex_coords = gl_FragCoord.xy / TargetSize;
	
	float luminance[8];
	for (int i = 0; i != 8; ++i) {
		vec2 uv = tex_coords + kernel[i] * 1.0 / TargetSize * sample_scale;
		luminance[i] = get_luminance(texture(InputTexture, uv).rgb);
	}
	
	float sum0 = luminance[7] + luminance[4] + luminance[5];
	float sum1 = luminance[2] + luminance[3] + luminance[0];
	float sum2 = luminance[2] + luminance[1] + luminance[7];
	float sum3 = luminance[0] + luminance[6] + luminance[5];
	
	// Amount of antialiasing: higher == more AA (too high causes artifacts)
	float filterStrength = 1;
	
	// subtract oposite sample set to get vector
	float vector1 = (sum1 - sum0) * filterStrength;
	float vector2 = (sum2 - sum3) * filterStrength;
	
	// combine vectors and multiply by pixel scale and normal scale
	vec2 normal = normal_scale * vec2(vector1, vector2) * 1.0 / TargetSize;
	
	vec3 Scene0 = texture(InputTexture, tex_coords).rgb;
	vec3 Scene1 = texture(InputTexture, tex_coords + normal.xy).rgb;
	vec3 Scene2 = texture(InputTexture, tex_coords - normal.xy).rgb;
	vec3 Scene3 = texture(InputTexture, tex_coords + vec2(normal.x, -normal.y)).rgb;
	vec3 Scene4 = texture(InputTexture, tex_coords - vec2(normal.x, -normal.y)).rgb;
	
	aa_out = (Scene0 + Scene1 + Scene2 + Scene3 + Scene4) * 0.2;
	
	// debug
	//aa_out.xyz = normalize(vec3(vector1, vector2 , 1) * 0.5 + 0.5);
}

*/

/*

// Tabula Rasa AA

uniform sampler2D InputTexture;
uniform vec2 TargetSize;

const vec2 dn_offsets[9] = vec2[9](
	vec2( 0.0,  0.0),
	vec2(-1.0, -1.0),
	vec2( 0.0, -1.0),
	vec2( 1.0, -1.0),
	vec2( 1.0,  0.0),
	vec2( 1.0,  1.0),
	vec2( 0.0,  1.0),
	vec2(-1.0,  1.0),
	vec2(-1.0,  0.0)
);

const vec2 c_offsets[4] = vec2[4](
	vec2(-0.5, -0.5),
	vec2( 0.5, -0.5),
	vec2( 0.5,  0.5),
	vec2(-0.5,  0.5)
);

float get_edge_weight(const in vec2 tex_coords) {
	
	float depth[9];
	vec3 normal[9];
	
	for (int i = 0; i != 9; ++i) {
		vec2 uv = tex_coords + dn_offsets[i] * 1.0 / TargetSize;
		
		normal[i]= g_get_vs_normal(uv);
		depth[i] = g_get_depth(uv);
	}
	
	vec4 delta_1 = vec4(depth[1], depth[2], depth[3], depth[4]);
	vec4 delta_2 = vec4(depth[5], depth[6], depth[7], depth[8]);
	
	delta_1 = abs(delta_1 - depth[0]);
	delta_2 = abs(depth[0] - delta_2);
	
	vec4 max_delta = max(delta_1, delta_2);
	vec4 min_delta = max(min(delta_1, delta_2), 0.00001);
	
	// magic number! - originally 25.0
	vec4 depthResults = step(min_delta * 5.0, max_delta);
	
	delta_1.x = dot(normal[1], normal[0]); delta_1.y = dot(normal[2], normal[0]);
	delta_1.z = dot(normal[3], normal[0]); delta_1.w = dot(normal[4], normal[0]);
	delta_2.x = dot(normal[5], normal[0]); delta_2.y = dot(normal[6], normal[0]);
	delta_2.z = dot(normal[7], normal[0]); delta_2.w = dot(normal[8], normal[0]);
	delta_1 = abs(delta_1 - delta_2);
	
	vec4 normalResults = step(0.4, delta_1);
	normalResults = max(normalResults, depthResults);
	
	return (normalResults.x + normalResults.y + normalResults.z + normalResults.w) * 0.25;
}

layout(location = 0) out vec3 aa_out;

void main() {
	vec2 tex_coords = gl_FragCoord.xy / TargetSize;
	
	float aa = min(get_edge_weight(tex_coords), 1.0);
	
	vec3 color = texture(InputTexture, tex_coords).rgb;
	
	if (aa != 0.0) {
		for (int i = 0; i != 4; ++i) {
			vec2 uv = tex_coords + c_offsets[i] * aa * 1.0 / TargetSize;
			color += texture(InputTexture, uv).rgb;
		}
		color /= 5.0;
	}
	
	aa_out = color;
}

*/

/*

// Custom AA

uniform sampler2D InputTexture;
uniform vec2 TargetSize;

const vec2 kernel[8] = vec2[8](
	vec2(-1.0, -1.0),
	vec2(-1.0,  0.0),
	vec2(-1.0,  1.0),
	vec2( 0.0, -1.0),
	vec2( 0.0,  1.0),
	vec2( 1.0, -1.0),
	vec2( 1.0,  0.0),
	vec2( 1.0,  1.0)
);

void get_deltas(const in vec2 tex_coords, out float deltas[8], out float edge, out float sum_delta) {
	
	vec3 normal = g_get_vs_normal(tex_coords);
	float depth = g_get_depth(tex_coords);
	
	float normal_deltas[8];
	float depth_deltas[8];
	
	float min_depth_delta = 0.0;
	float max_depth_delta = 0.0;
	
	for (int i = 0; i != 8; ++i) {
		vec2 uv = tex_coords + kernel[i] * 0.5 / TargetSize;
		
		vec3 normal_i = g_get_vs_normal(uv);
		float depth_i = g_get_depth(uv);
		
		normal_deltas[i] = abs(1.0 - (dot(normal_i, normal) * 0.5 + 0.5));
		depth_deltas[i] = abs(depth - depth_i);
		
		max_depth_delta = max(max_depth_delta, depth_deltas[i]);
		min_depth_delta = min(min_depth_delta, depth_deltas[i]);
	}
	
	// Magic numbers!
	float min_depth_step = 50.0 * max(min_depth_delta, 0.00001);
	float max_depth_step = 100.0 * max(min_depth_delta, 0.00001);
	
	edge = 0.0;
	sum_delta = 0.0;
	
	for (int i = 0; i != 8; ++i) {
		float final_depth_delta = smoothstep(min_depth_step, max_depth_step, depth_deltas[i]);
		float final_normal_delta = smoothstep(0.4, 1.0, normal_deltas[i]);
		deltas[i] = mix(final_depth_delta, final_normal_delta, 0.5);
		
		edge += final_depth_delta + final_normal_delta;
		sum_delta += 1.0 - deltas[i];
	}
}

layout(location = 0) out vec3 aa_out;

void main() {
	vec2 tex_coords = gl_FragCoord.xy / TargetSize;
	
	float[8] deltas;
	float edge, sum_delta;
	get_deltas(tex_coords, deltas, edge, sum_delta);
	
	vec3 color = texture(InputTexture, tex_coords).rgb;
	
	if (edge != 0.0) {
		for (int i = 0; i != 8; ++i) {
			vec2 uv = tex_coords + kernel[i] * 1.0 / TargetSize;
			color += texture(InputTexture, uv).rgb * (1.0 - deltas[i]);
		}
		color /= sum_delta + 1.0;
	}
	
	aa_out = color;
}

*/