
uniform vec2 TargetSize;
uniform mat4 InvProjMatrix;

uniform vec3 LightColor;

uniform float CosInnerCutOff, CosOuterCutOff;

uniform vec3 VSLightPosition, VSLightDirection;
uniform float VSLightRadius;

const float two_sided = 0.0;

layout(location = 0) out vec3 light_out;

void main() {
	vec2 tex_coords = gl_FragCoord.xy / TargetSize;
	
	float depth = g_get_depth(tex_coords);
	vec3 p = g_get_vs_position(tex_coords, depth, InvProjMatrix);
	
	vec3 v = normalize(-p);
	vec3 l = VSLightPosition - p;
	float d = length(l);
	
	if (d > VSLightRadius)
		discard;
	
	l /= d;
	
	float l_dot_dir = dot(-l, normalize(VSLightDirection));
	float spot_attenuation = smoothstep(CosOuterCutOff, CosInnerCutOff, l_dot_dir);
	float attenuation = 1.0 - smoothstep(0.0, 1.0, d / VSLightRadius);
	
	vec3 n = g_get_vs_normal(tex_coords);
	
	vec3 h = normalize(l + v);
	
	float n_dot_h = dot(n, h);
	n_dot_h = n_dot_h < 0.0 ? two_sided * -n_dot_h : n_dot_h;
	
	float n_dot_l = dot(n, l);
	n_dot_l = n_dot_l < 0.0 ? two_sided * -n_dot_l : n_dot_l;
	
	vec3 diffuse = g_get_diffuse(tex_coords);
	
	vec3 C = (diffuse + diffuse * pow(n_dot_h, 100.0)) * LightColor * n_dot_l * attenuation * spot_attenuation;
	
	light_out = C;
}
