
void integrand(in float r, in float mu, in float muS, in float nu, in float t, out vec3 ray, out vec3 mie) {
    float ri = sqrt(r * r + t * t + 2.0 * r * mu * t);
    float muSi = (nu * t + muS * r) / ri;
    ri = max(Rg, ri);
	
    if (muSi >= -sqrt(1.0 - Rg * Rg / (ri * ri))) {
        vec3 ti = transmittance(r, mu, t) * transmittance(ri, muSi);
		
        ray = exp(- (ri - Rg) / HR) * ti;
        mie = exp(- (ri - Rg) / HM) * ti;
    }
	else {
		ray = vec3(0.0);
		mie = vec3(0.0);
	}
}

void inscatter(in float r, in float mu, in float muS, in float nu, out vec3 ray, out vec3 mie) {
    float dx = limit(r, mu) / float(INSCATTER_INTEGRAL_SAMPLES);
	
    vec3 rayi, miei;
    integrand(r, mu, muS, nu, 0.0, rayi, miei);
	
    ray = vec3(0.0);
	mie = vec3(0.0);
	
    for (int i = 1; i <= INSCATTER_INTEGRAL_SAMPLES; i++) {
		vec3 rayj, miej;
        integrand(r, mu, muS, nu, float(i) * dx, rayj, miej);
		
        ray += (rayi + rayj) / 2.0 * dx;
        mie += (miei + miej) / 2.0 * dx;
        
		rayi = rayj;
        miei = miej;
    }
	
    ray *= betaR;
    mie *= betaMSca;
}

flat in float r;
flat in vec4 dhdH;

layout(location = 0) out vec3 rayleigh_out;
layout(location = 1) out vec3 mie_out;

void main() {
    float mu, muS, nu;
    getMuMuSNu(r, dhdH, mu, muS, nu);
	
    // store Rayleigh and Mie contributions separately, WITHOUT the phase function factor
    inscatter(r, mu, muS, nu, rayleigh_out, mie_out);
}
