
uniform sampler3D DeltaJTexture;

vec3 integrand(in float r, in float mu, in float muS, in float nu, in float t) {
    float ri = sqrt(r * r + t * t + 2.0 * r * mu * t);
    float mui = (r * mu + t) / ri;
    float muSi = (nu * t + muS * r) / ri;
    return texture4D(DeltaJTexture, ri, mui, muSi, nu).rgb * transmittance(r, mu, t);
}

void inscatter(in float r, in float mu, in float muS, in float nu, out vec3 raymie) {
    raymie = vec3(0.0);
	
    float dx = limit(r, mu) / float(INSCATTER_INTEGRAL_SAMPLES);
    vec3 raymiei = integrand(r, mu, muS, nu, 0.0);
	
    for (int i = 1; i <= INSCATTER_INTEGRAL_SAMPLES; ++i) {
        vec3 raymiej = integrand(r, mu, muS, nu, float(i) * dx);
        raymie += (raymiei + raymiej) / 2.0 * dx;
        raymiei = raymiej;
    }
}

flat in float r;
flat in vec4 dhdH;

layout(location = 0) out vec3 inscatter_out;

void main() {
    float mu, muS, nu;
    getMuMuSNu(r, dhdH, mu, muS, nu);
	
    inscatter(r, mu, muS, nu, inscatter_out);
}
