
noperspective in vec3 ray;
noperspective in vec3 dist;

uniform vec2 WindowSize;

// todo: make uniform
const float ISun = 100.0;
const float exposure = 0.4;

uniform vec3 cam_pos, sun_dir;

uniform sampler3D InscatterTexture;
uniform sampler2D IrradianceTexture;

vec3 sky_inscatter(in vec3 x, in vec3 v, in vec3 s, in float r, in float mu) {
	// Assume x is not in space, and ray intersects the atmosphere
	
	float nu = dot(v, s);
	float muS = dot(x, s) / r;
	float phaseR = phaseFunctionR(nu);
	float phaseM = phaseFunctionM(nu);
	vec4 inscatter = max(texture4D(InscatterTexture, r, mu, muS, nu), 0.0);
	
	// avoids imprecision problems in Mie scattering when sun is below horizon
	inscatter.w *= smoothstep(0.00, 0.02, muS);
	
	vec3 result = max(inscatter.rgb * phaseR + getMie(inscatter) * phaseM, 0.0);
	
    return result * ISun;
}

// direct sun light for ray x+tv, when sun in direction s (=L0)
vec3 sun_light(in vec3 x, in vec3 v, in vec3 s, in float r, in float mu) {
	vec3 transmittance = vec3(1.0); // T(x,xo)
	if (r <= Rt) {
		transmittance = transmittanceWithShadow(r, mu);
	}
	
	float isun = step(cos(M_PI / 180.0), dot(v, s)) * ISun; // Lsun
	return transmittance * isun; // Eq (9)
}

vec3 HDR(in vec3 L) {
    L = L * exposure;
    L.r = L.r < 1.413 ? pow(L.r * 0.38317, 1.0 / 2.2) : 1.0 - exp(-L.r);
    L.g = L.g < 1.413 ? pow(L.g * 0.38317, 1.0 / 2.2) : 1.0 - exp(-L.g);
    L.b = L.b < 1.413 ? pow(L.b * 0.38317, 1.0 / 2.2) : 1.0 - exp(-L.b);
    return L;
}

layout(location = 0) out vec3 sky_out;

void main()
{
	vec2 tex_coords = gl_FragCoord.xy / WindowSize;
	
	if (g_get_object_type(tex_coords) != g_TYPE_NOTHING) {
		discard;
	}
	
	vec3 offset = vec3(0.0, 0.0, Rg);
	
	// world space camera pos (in km, not m)
	vec3 x = cam_pos.xzy / 1000.0 + offset;
	
	// normalize ray dir
    vec3 v = normalize(ray.xzy);
	
    float r = length(x);
    float mu = dot(x, v) / r;
	
	vec3 s = normalize(sun_dir.xzy);
	
	vec3 color = vec3(0.0);
	
	color += sky_inscatter(x, v, s, r, mu); // S[L]-T(x,xs)S[l]|xs
	color += sun_light(x, v, s, r, mu); // L0
	
    sky_out = HDR(color);
}


