
// From http://cgg-journal.com/2008-2/06/index.html (Shader Based Wireframe Drawing)

// output triangles (quads from tessellation are subdivided to tris)
layout(triangles) in;
layout(triangle_strip, max_vertices = 3) out;

uniform vec2 WindowSize;

noperspective out vec3 dist;

// position is used to create detail texture coordinates
in vec3[] VPosition;
out vec3 Position;

// used to read alpha maps
in vec2[] VTexCoords;
out vec2 TexCoords;

// depth for g_buffer
in vec2[] VDepthPos;
out vec2 DepthPos;

void main()
{
	vec2 p0 = WindowSize * gl_in[0].gl_Position.xy / gl_in[0].gl_Position.w;
	vec2 p1 = WindowSize * gl_in[1].gl_Position.xy / gl_in[1].gl_Position.w;
	vec2 p2 = WindowSize * gl_in[2].gl_Position.xy / gl_in[2].gl_Position.w;
	
	vec2 v0 = p2 - p1;
	vec2 v1 = p2 - p0;
	vec2 v2 = p1 - p0;
	float area = abs(v1.x * v2.y - v1.y * v2.x);
	
	
	Position = VPosition[0];
	TexCoords = VTexCoords[0];
	DepthPos = VDepthPos[0];
	dist = vec3(area / length(v0), 0, 0);
	
	gl_Position = gl_in[0].gl_Position;
	EmitVertex();
	
	
	Position = VPosition[1];
	TexCoords = VTexCoords[1];
	DepthPos = VDepthPos[1];
	dist = vec3(0, area / length(v1), 0);
	
	gl_Position = gl_in[1].gl_Position;
	EmitVertex();
	
	
	Position = VPosition[2];
	TexCoords = VTexCoords[2];
	DepthPos = VDepthPos[2];
	dist = vec3(0, 0, area / length(v2));
	
	gl_Position = gl_in[2].gl_Position;
	EmitVertex();
	
	
	EndPrimitive();
}
