
// subdivide into quads
layout(vertices = 4) out;

// Uses the technique described here:
// https://developer.nvidia.com/content/dynamic-hardware-tessellation-basics

uniform mat4 ModelViewProjectionMatrix;
uniform float ProjectionMatrix11;

float sphere_extent(vec3 origin, float diameter)
{
	vec4 projOrigin = ModelViewProjectionMatrix * vec4(origin, 1.0);
	
	return abs(diameter * ProjectionMatrix11 / projOrigin.w);
}

const float MinTessLevel = 1.0;
const float MaxTessLevel = 64.0;

uniform float EdgesPerScreenHeight;

float level(vec3 v1, vec3 v2)
{
	float d = distance(v1, v2);
		
	vec3 mid = (v1 + v2) / 2.0;
	
	float factor = sphere_extent(mid, d);
	
	return clamp(factor * EdgesPerScreenHeight, MinTessLevel, MaxTessLevel);
}

void main()
{
	if (gl_InvocationID == 0)
	{
		vec3 v0 = gl_in[0].gl_Position.xyz;
		vec3 v1 = gl_in[1].gl_Position.xyz;
		vec3 v2 = gl_in[2].gl_Position.xyz;
		vec3 v3 = gl_in[3].gl_Position.xyz;
		
		float l0 = level(v0, v3);
		float l1 = level(v0, v1);
		float l2 = level(v1, v2);
		float l3 = level(v2, v3);
		
		gl_TessLevelInner[0] = mix(l0, l2, 0.5);
		gl_TessLevelInner[1] = mix(l1, l3, 0.5);
		gl_TessLevelOuter[0] = l0;
		gl_TessLevelOuter[1] = l1;
		gl_TessLevelOuter[2] = l2;
		gl_TessLevelOuter[3] = l3;
	}
	
	gl_out[gl_InvocationID].gl_Position = gl_in[gl_InvocationID].gl_Position;
}
