
declare_globals{
	"tprint",
	"tmerge",
	"tdiff",
}

local function serialize(v)
	if type(v) == "string" then
		return string.format("%q", v)
	end
	return tostring(v)
end

function tprint(key, value, saved)
	saved = saved or {}
	local t = type(value)
	if t == "table" then
		if saved[value] then
			return key.." = "..saved[value].."\n"
		else
			saved[value] = key
			local s = key.." = ".."{}\n"
			for k,v in pairs(value) do
				k = serialize(k)
				local n = string.format("%s[%s]", key, k)
				local st = tprint(n, v, saved) or ""
				s = s .. st
			end
			return s
		end
	elseif t == "number" or t == "string" or t == "boolean" then
		return key.." = "..serialize(value).."\n"
	else
		return "-- "..key.." = "..serialize(value).."\n"
	end
end

-- Merge table t into table dest.
function tmerge(dest, t)
	if type(dest) ~= "table" or type(t) ~= "table" then
		error("Call to tmerge with non-table values.")
	end
	for k,v in pairs(t) do
		if not dest[k] or type(dest[k]) ~= type(v) then
			if type(v) == "table" then
				local dv = {}
				tmerge(dv, v)
				dest[k] = dv
			else
				dest[k] = v
			end
		else
			if type(v) == "table" then
				tmerge(dest[k], v)
			else
				dest[k] = v
			end
		end
	end
end

-- Removes values from diff that are also in t.
function tdiff(diff, t)
	if type(diff) ~= "table" or type(t) ~= "table" then
		error("Call to tdiff with non-table values.")
	end
	local i = 0
	for k,v in pairs(diff) do
		if type(v) == type(t[k]) and v == t[k] then
			diff[k] = nil
		else
			i = i + 1
		end
	end
	return i
end
